package gui;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Observable;
import java.util.Vector;
@SuppressWarnings("serial") 

/**
 * Data Bean for the CSI Application. It has fields for the currentDBName, current DBPath, currentCSIName and currentCSIEmail.
 * When the selected database is changed, the model fields are updated.
 * When a CSI logs it, this also finds out the databases the CSI is monitoring by querying the isMaintainedBy relation.
 * The Database configuration is also loaded and updated as requested by the user.
 * 
 * @author Natasha Gaitonde - gaitonde@email.arizona.edu
 *
 */
public class CSIModel extends Observable
{
	private String currentCSIName = "";
	private String currentCSIEmail = "";
	private String currentDBName = "";
	private String currentDBPath = "";
	private boolean updated = false, overall=true;

	public void setSelectedDB(String dbName, String dbPath)
	{
		currentDBName = dbName;
		currentDBPath = dbPath;
		this.setChanged();
		this.notifyObservers();
	}

	public String getSelectedDB()
	{
		return currentDBName;
	}

	public void setCurrentCSI(String email)
	{
		currentCSIEmail = email;
	}

	//Find which databases this CSI is Managing
	public Vector getManagedDatabases()
	{
		if(currentCSIName == null || currentCSIEmail == null)
		{
			return new Vector();
		}
		Vector list = new Vector();
		Connection conn = Connector.getConnection();
		String query = "select Name, Path from IsMaintainedBy where Email='" + currentCSIEmail + "' and Kind='C' and StopDate is null;";
		if(conn != null)
		{
			try
			{
				Statement stmt = conn.createStatement();
				ResultSet rset = stmt.executeQuery(query);
				if(rset.next())
				{
					currentDBName = rset.getString(1);//first db is set as currentDBName
					currentDBPath = rset.getString(2);
					list.add(currentDBName.trim() + "  (" + currentDBPath.trim() + ")");
				}
				while(rset.next())
				{
					list.add(rset.getString(1).trim() + "  (" + rset.getString(2).trim() + ")");
				}
				conn.close();
				return list;
			}
			catch (SQLException se)
			{
				se.printStackTrace();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return new Vector();
	}

	//Find the DBA for the databases managed by the current CSI
	public String getCurrentDBA()
	{
		String email = "";
		Connection conn = Connector.getConnection();
		String query = "select Email from IsMaintainedBy where Name = '" + this.currentDBName + "' and Path='" + this.currentDBPath + "' and Kind='D' and StopDate is null;";
		if(conn != null)
		{
			try
			{
				Statement stmt = conn.createStatement();
				ResultSet rset = stmt.executeQuery(query);
				if(rset.next())
				{
					email = rset.getString(1);
					query = "Select Name from Employee where Email='" + email + "';";
					stmt = conn.createStatement();
					rset = stmt.executeQuery(query);
					if(rset.next())
					{
						return rset.getString(1) + "(" + email + ")";
					}
				}
				conn.close();
			}
			catch (SQLException se)
			{
				se.printStackTrace();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return "";
	}
	public String getCurrentDBName() 
	{
		return currentDBName;
	}
	public void setCurrentDBName(String currentDBName) 
	{
		this.currentDBName = currentDBName;
	}
	public String getCurrentDBPath() 
	{
		return currentDBPath;
	}
	public void setCurrentDBPath(String currentDBPath) 
	{
		this.currentDBPath = currentDBPath;
	}
	
	//Update the configuration table
	public int setConfig(DatabaseConfig config, DatabaseConfig confignew)
	{
		String query = null;
		int refresh=0;
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
		updated=config.equalsObj(confignew);//Determine if the configuration has been changed since loading it
		if(updated==true)
		{
			//Configuration has changed, need to update or insert a tuple
			if (overall==true)//Overall config was displayed initially, so now insert a new tuple for this dbName
			{
				Calendar cal = Calendar.getInstance();
				query = new String("insert into Configuration (StoredDate,NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path) values ('" 
						+ new Timestamp(cal.getTime().getTime()) + "'," + confignew.getNotzIntervalDays() + ", " + confignew.getNotzIntervalHours() + ", " + confignew.getNotzIntervalMins() + ", " 
						+ confignew.getValidationFactor() + ", '" + confignew.getAlgorithm().trim() + "'," + confignew.getLevel() + ", '" + this.currentDBName.trim() + "', '" + this.currentDBPath.trim() + "');");
				overall=false;
			}
			else
			{
				//update the existing configuration for the dbName
				try
				{
					String tmpdate= sdf.format(config.getStoredDate());
				query = new String("update Configuration set NotIntDays=" + confignew.getNotzIntervalDays() + ", NotIntHrs=" + confignew.getNotzIntervalHours() + ", " +
						"NotIntMins=" + confignew.getNotzIntervalMins() + ", ValidationFactor=" + confignew.getValidationFactor() + ", Algorithm='" + confignew.getAlgorithm().trim() + "'," +
						"NumLevels=" + confignew.getLevel() + " where Name='" + this.currentDBName.trim() + "' and StoredDate='" + tmpdate + "'");
				refresh=1;
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
			Connection conn = Connector.getConnection();
			if(conn != null)
			{
				try
				{
					Statement stmt = conn.createStatement();
					stmt.executeUpdate(query);
				}
				catch(Exception e)
				{
					e.printStackTrace();
				}
			}
		}
		return refresh;
	}
 
    // Add getAllFailures() by Michael;
    public Vector getAllFailures()
    {
        Vector<String> list = new Vector<String>();


        Connection conn = Connector.getConnection();
        String query = "select Name, ActualDate from Event where Kind='T';";


        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    list.add("Tamper detection in \"" + rset.getString(1) + "\" on " + rset.getString(2) + ".");
                }

                conn.close();

                return list;

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get all failures");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get all failures");
                System.err.println(e.toString());
            }
        }

        return list;
    }








	//Retrieve the configuration for the dbName. If no configuration exists for the specific dbName then return the most 
	//recent overall configuration
	public DatabaseConfig getCurrentConfig()
	{
		Connection conn = Connector.getConnection();
		String query = "SELECT NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, StoredDate FROM Configuration WHERE Name = '" + this.currentDBName + "' ORDER BY StoredDate DESC LIMIT 1;";
		if(conn != null)
		{
			try
			{
				Statement stmt = conn.createStatement();
				ResultSet rset = stmt.executeQuery(query);
				if(rset.next())
				{
					//this dbName has a configuration tuple in the database
					overall=false;
					return new DatabaseConfig(this.currentDBName, rset.getString(5), rset.getInt(1), rset.getInt(2), rset.getInt(3), rset.getInt(4), rset.getInt(6),rset.getTimestamp(7),null);
				}
				else
				{
					//no individual configuration for this dbName. Retrieve the most recent overall configuration
					overall=true;
					query = null;
					query = new String("SELECT NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path, StoredDate FROM Configuration WHERE Name = 'Overall' ORDER BY StoredDate DESC LIMIT 1;");
					rset = stmt.executeQuery(query);
					if(rset.next())
					{
						return new DatabaseConfig("Overall", rset.getString(5), rset.getInt(1), rset.getInt(2), rset.getInt(3), rset.getInt(4), rset.getInt(6),rset.getTimestamp(9),"0000-00-00 00:00:00");
					}   
				}
				conn.close();
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		return new DatabaseConfig();
	}
}
