package gui;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.sql.ResultSet;
import java.sql.Timestamp;
import java.util.HashMap;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
@SuppressWarnings("serial") 

/**
 * Screen for displaying the Database events plot. The Corruption Diagram is shown. 
 * Notarization events, Validation events and Corruption region(s) (if any) are plotted by retrieving the timestamps
 * from the database.
 * 3 files are created for plotting. One contains timestamps of Notarization events, one for Validation events
 * and the last one is the actual GNUPlot script that is generated dynamically. This is helpful for zooming in and out.
 * 
 * Note - Corruption regions can be plotted as rectanges ONLY in GNUPlot 4.2
 * @author Natasha Gaitonde - gaitonde@email.arizona.edu
 *
 */

public class CSIPlotMain extends JFrame
{
	private String command;
	private JLabel imageLabel = new JLabel(); 
	CSIPlotDB zdbn = new CSIPlotDB();
	private String dbname = null;
	private String dbpath = null;	
	private Timestamp firstts = null;
	private Timestamp lastts = null;
	CSITSWrapper cwfirstlast;
	private JButton zobtn = new JButton("Zoom Out");
	private CSIZOBTN czbtn;
	private CSIUtil tsutil;
	private CSIMAC csiMac;
	int zilevel=-1;
	HashMap<Integer, CSITSWrapper> zo = new HashMap<Integer, CSITSWrapper>();//HashMap to store previous zoom ins so that we can zoom out to them later
	CSITSWrapper zi;//Store the current zoom in object onlyl
	CSITSWrapper cwpt;
	CSIPlotMain(String dbname, String dbpath)
	{
		command = new String("/usr/local/bin/gnuplot " + CSIConstants.gnuPlotScript);//Path for GNUPlot 4.2 on Lectura
		this.dbname=dbname;
		this.dbpath=dbpath;
		this.tsutil = new CSIUtil();
		cwpt = new CSITSWrapper();
		cwfirstlast = new CSITSWrapper();
		zi =  new CSITSWrapper();
		this.setResizable(false);
		czbtn = new CSIZOBTN(this);
		zobtn.addActionListener(czbtn);//Button at the top for zoom out		
		zobtn.setSize(20,30);
		csiMac = new CSIMAC(this);
		this.addMouseListener(csiMac);		
	}	
	public void drawEvents() 
	{
		genFiles();	//generate files for Notarization and Validation Timestamps	
		gnuPlotScript(0,0);//Generate GNUPlot script
		drawImage();//Display the corruption diagram
		return;
	}
	
	/**
	 * Method to read the database and generate 2 files of timestamps for Notarization and Validation events
	 *
	 */
	void genFiles()
	{
		BufferedWriter f1;
		BufferedWriter f2;
		ResultSet rs = null;		
		long diff;
		try
		{
			f1=new BufferedWriter(new FileWriter(CSIConstants.notFile));
			f2=new BufferedWriter(new FileWriter(CSIConstants.valFile));
			
			rs = zdbn.findActualDate(-1,this.dbname, this.dbpath, "C");//get creation time
			if(rs.next())
			{
				firstts = new Timestamp(rs.getTimestamp(1).getTime());
				cwfirstlast.xlow = new Timestamp(firstts.getTime());
				cwfirstlast.ylow = cwfirstlast.getXlow();
				f1.write(tsutil.formattedTS(rs.getTimestamp(1)) + "\n"); //This should be the first notarization but it's not!
			}					
			rs = zdbn.findActualDate(-1,this.dbname, this.dbpath, "N");//get Notarization timestamps
			while(rs.next())
			{
				f1.write(tsutil.formattedTS(rs.getTimestamp(1)) + "\n");
			}			
			rs = zdbn.findActualDate(-1,this.dbname, this.dbpath, "V");//get Validation timestamps
			while(rs.next())
			{
				diff = (rs.getTimestamp(1).getTime()-firstts.getTime())/1000;
				f2.write(tsutil.formattedTS(firstts) + " " + tsutil.formattedTS(rs.getTimestamp(1)) + " " +String.valueOf(diff) + " 0\n");
				lastts = new Timestamp(rs.getTimestamp(1).getTime());
			}
			lastts.setMinutes(lastts.getMinutes()+1);
			cwfirstlast.xhigh = new Timestamp(lastts.getTime());
			cwfirstlast.yhigh = cwfirstlast.getXhigh();//store the timestamps of the first and last Notarization events
			cwpt.makeEqual(cwfirstlast);
			zi.makeEqual(cwpt);
			f1.close();
			f2.close();	
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}	
	}
	
	/**
	 * This method generates the actual GNUPlot script in a file whose name is in CSIConstants.gnuPlotScript
	 * This file is created in the current directory. It has conditional contents for plotting the corruption region, if 
	 * any are present and for setting the xrange and yrange for zooming in and out
	 * @param level is the Zoom in depth
	 * @param quad is the quadrant for zooming in
	 */
	public void gnuPlotScript(int level, int quad)
	{
		StringBuffer data = new StringBuffer();
		BufferedWriter f1;
		HashMap hmCorruption = new HashMap();
		ResultSet rs = null;
		CSIPlotCTS zcts = new CSIPlotCTS();
		CSIPlotCorruption zc = new CSIPlotCorruption();
		hmCorruption.clear();
		try
		{
			hmCorruption=zdbn.findCorruptions(this.dbname,this.dbpath);//populate corruption region HashMap
			f1 = new BufferedWriter(new FileWriter(CSIConstants.gnuPlotScript));
			//Some parts of the script file that are static
			data.append("set terminal png\n" +
					"set output '" + CSIConstants.outputImage+"'\n" +
					"set title 'Corruption Diagram'\n" +
					"set xlabel 'Where'\n" +
					"set ylabel 'When'\n" +
					"set xdata time\n" +
					"set ydata time\n" +
					"set timefmt '%Y-%m-%d:%H:%M:%S'\n" +
					"set format x '%M:%S'\n" +
					"set format y '%Y-%m-%d %H:%M:%S'\n" +
					"set key right bottom\n");
			//for each corruption region, append a line in the script to plot the rectangle
			for(int i=0; i <hmCorruption.size();i++)
			{
				zc=(CSIPlotCorruption)hmCorruption.get(new Integer(i));
				rs = zdbn.findActualDate(zc.getBottomLeftX(),this.dbname,this.dbpath,"");
				while(rs.next())
				{
					zcts.setBotlx(rs.getTimestamp(1));
				}	
				rs = zdbn.findActualDate(zc.getBottomLeftY(),this.dbname,this.dbpath,"");
				while(rs.next())
				{
					zcts.setBotly(rs.getTimestamp(1));
				}	
				rs = zdbn.findActualDate(zc.getTopRightX(),this.dbname,this.dbpath,"");
				while(rs.next())
				{
					zcts.setToprx(rs.getTimestamp(1));
				}	
				rs = zdbn.findActualDate(zc.getTopRightY(),this.dbname,this.dbpath,"");
				while(rs.next())
				{
					zcts.setTopry(rs.getTimestamp(1));
				}	
				data.append("set object rect from '" + tsutil.formattedTS(zcts.getBotlx()) + "','" + tsutil.formattedTS(zcts.getBotly()) + "' to '" + tsutil.formattedTS(zcts.getToprx()) + "','" + tsutil.formattedTS(zcts.getTopry()) + "' fc lt 1\n");
			}
			if(level>0 && quad==0)//zoom out
			{
				CSITSWrapper tmp = zo.get(new Integer(zilevel));
				zilevel--;
				cwpt.makeEqual(tmp);
				zi.makeEqual(cwpt);
			}
			else
			{
				//zoom in
				zilevel++;
				if(level>0 && quad>0)
				{
					CSITSWrapper zocw = new CSITSWrapper(zi.xlow,zi.xhigh,zi.ylow,zi.yhigh);
					zo.put(new Integer(zilevel),zocw);
					
					//Reset low and high, x and y values accordingly for replotting and in the hashmap
					switch(quad)
					{
					case 1://1st quadrant change xhigh and yhigh
					{
						cwpt.setXhigh(cwpt.xlow.getTime()+(cwpt.xhigh.getTime()-cwpt.xlow.getTime())/2);
						cwpt.setYhigh(cwpt.ylow.getTime()+(cwpt.yhigh.getTime()-cwpt.ylow.getTime())/2);						
						break;
					}
					case 2://2nd quadrant change xlow and yhigh
					{
						cwpt.setXlow(cwpt.xlow.getTime()+(cwpt.xhigh.getTime()-cwpt.xlow.getTime())/2);
						cwpt.setYhigh(cwpt.ylow.getTime()+(cwpt.yhigh.getTime()-cwpt.ylow.getTime())/2);
						break;
					}
					case 3://3rd quadrant change xhigh and ylow
					{
						cwpt.setXhigh(cwpt.xlow.getTime()+(cwpt.xhigh.getTime()-cwpt.xlow.getTime())/2);
						cwpt.setYlow(cwpt.ylow.getTime()+(cwpt.yhigh.getTime()-cwpt.ylow.getTime())/2);
						break;
					}
					case 4://4th quadrant change xlow and ylow
					{
						cwpt.setXlow(cwpt.xlow.getTime()+(cwpt.xhigh.getTime()-cwpt.xlow.getTime())/2);
						cwpt.setYlow(cwpt.ylow.getTime()+(cwpt.yhigh.getTime()-cwpt.ylow.getTime())/2);
						break;
					}
					}	
					zi.makeEqual(cwpt);
				}		
			}
			//dynamic contents for range
			data.append("set xrange ['" + tsutil.formattedTS(cwpt.xlow) + "':'" + tsutil.formattedTS(cwpt.xhigh) + "']\n");
			data.append("set yrange ['" + tsutil.formattedTS(cwpt.ylow) + "':'" + tsutil.formattedTS(cwpt.yhigh) + "']\n");
			data.append("plot '" + CSIConstants.notFile + "' using 1:1 with linespoints lt 2 linewidth 2 title 'Notarization Events'," +
					"'" + CSIConstants.valFile + "' using 1:2:3:4 with vector nohead lt 1 linewidth 2 title 'Validation Events'\n");
			f1.write(data.toString());
			f1.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}		
	}	
	
	/**
	 * Execute the GNUPlot script and display the generated png file on the screen
	 *
	 */
	void drawImage()
	{
		File f;
		BufferedImage bi;
		ImageIcon ic;
		Toolkit kit = Toolkit.getDefaultToolkit();
		Dimension screenSize = kit.getScreenSize();
		int screenHt = screenSize.height;
		int screenWt = screenSize.width;
		try
		{
			Process p = Runtime.getRuntime().exec(command);//Execute gnuplot command
			p.waitFor();
			p.destroy();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		try
		{
			f = new File(CSIConstants.outputImage);
			bi = ImageIO.read(f);//Read the generated PNG file
			ic = new ImageIcon(bi);
			imageLabel.setIcon(ic);
			imageLabel.setSize(screenWt,screenHt);
			this.getContentPane().add(zobtn,BorderLayout.NORTH);
			this.getContentPane().add(imageLabel,BorderLayout.SOUTH);
			this.getContentPane().invalidate();
			this.setLocation(screenWt/4,screenHt/4);
			this.setVisible(true);	
			this.pack();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}		
}