package gui;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.sql.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.Date;



/**
 * Maintains the state of the CSO GUI. Interacts with the database to update and
 * retrieve information about the system. 
 * 
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
public class CSOModel extends Observable
{
    //Turn on for debugging
    private boolean debug = false;

    
    private String notarize_server_;
    private String dbcreate_time_getter_;
    
    public void setNotarizeServer(String notarize_server) {
      notarize_server_ = notarize_server;
    }
    
    public void setDBCreateTimeGetter(String dbcreate_time_getter) {
    	dbcreate_time_getter_ = dbcreate_time_getter;    	
    }

    /**
     * Sets the current CSI of the given database.
     * 
     * @param dbName the name of the database.
     * @param dbPath the path of the database.
     * @param csiName the name of the new CSI.
     * @param csiEmail the email of the new CSI.
     */
    public String setCurrentCSI(String dbName, String dbPath, String csiName, String csiEmail)
    {
        String query;
        String csi = getCurrentCSI(dbName, dbPath);

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                //If this is not the first CSI assigned to this database we need to update the old CSI's record with the end date
                if(csi != null)
                {
                    //If the CSI is the same then we can return
                    if(csi.equals(csiName + "(" + csiEmail + ")"))
                    {
                        return "";  //Success
                    }

                    DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
                    DateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
                    Date currentDate = new Date();

                    query = "update IsMaintainedBy set StopDate = '" + dateFormatter.format(currentDate) + " " + timeFormatter.format(currentDate) + "' where Name='" + dbName + "' and Path='" + dbPath + "' and Kind = 'C' and StopDate is null;";
                    if(debug) { System.out.println(query); };
                    Statement stmt = conn.createStatement();
                    stmt.executeUpdate(query);
                }

                query = "insert into IsMaintainedBy (Name, Path, Email, Kind) values ('" + dbName + 
                "', '" + dbPath + "', '" + csiEmail + "', 'C');";
                if(debug) { System.out.println(query); };   
                Statement stmt = conn.createStatement();
                stmt.executeUpdate(query);

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to set CSI for " + dbName);
                System.err.println(se.toString());
                return "There was an error while attempting to make this assignment. The assignment was NOT made.";
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to set CSI for " + dbName);
                return "There was an error while attempting to make this assignment. The assignment was NOT made.";
            }
        }

        return "";  //Success
    }


    /**
     * Gets the current CSI for the given database.
     * 
     * @param dbName the name of the database
     * @param dbPath the path of the database
     * @return the CSI of the given database in the form: Name(email)
     */
    public String getCurrentCSI(String dbName, String dbPath)
    {
        String email = "";
        Connection conn = Connector.getConnection();
        String query = "select Email from IsMaintainedBy where Name = '" + dbName + "' and Path='" + dbPath + "' and Kind='C' and StopDate is null;";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {
                    email = rset.getString(1);
                    query = "Select Name from Employee where Email='" + email + "';";
                    stmt = conn.createStatement();
                    rset = stmt.executeQuery(query);

                    if(rset.next())
                    {
                        return rset.getString(1) + "(" + email + ")";
                    }
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to retrieve a CSI assignment.");
                System.err.println(se.toString());
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to retrieve a CSI assignment.");
            }
        }
        return null;
    }


    /**
     * Resets the given database's CSI so that the database does not have a CSI assigned to it.
     * 
     * @param dbName the name of the database.
     * @param dbName the path of the database.
     */
    public void clearCSI(String dbName, String dbPath)
    {
        String query;
        String csi = getCurrentCSI(dbName, dbPath);

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                //If this is not the first CSI assigned to this database we need to update the old CSI's record with the end date
                if(csi != null)
                {
                    DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
                    DateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
                    Date currentDate = new Date();

                    query = "update IsMaintainedBy set StopDate = '" + dateFormatter.format(currentDate) + " " + timeFormatter.format(currentDate) + "' where Name='" + dbName +"' and Path='" + dbPath + "' and Kind = 'C' and StopDate is null;";
                    if(debug) { System.out.println(query); };

                    Statement stmt = conn.createStatement();
                    stmt.executeUpdate(query);
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to clear CSI for " + dbName);
                System.err.println(se.toString());
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to clear CSI for " + dbName);
            }
        }
    }


    /**
     * Sets the current DBA for the given database.
     * 
     * @param dbName the name of the database.
     * @param dbPath the path of the database.
     * @param dbaName the name of the new DBA.
     * @param dbaEmail the email of the new DBA.
     */
    public String setCurrentDBA(String dbName, String dbPath, String dbaName, String dbaEmail)
    {
        String query;
        String dba = getCurrentDBA(dbName, dbPath);

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                //If this is not the first CSI assigned to this database we need to update the old CSI's record with the end date
                if(dba != null)
                {
                    //If the CSI is the same then we can return
                    if(dba.equals(dbaName + "(" + dbaEmail + ")"))
                    {
                        return "";  //Success
                    }

                    DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
                    DateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
                    Date currentDate = new Date();

                    query = "update IsMaintainedBy set StopDate = '" + dateFormatter.format(currentDate) + " " + timeFormatter.format(currentDate) + "' where Name='" + dbName +"' and Path='" + dbPath + "' and Kind = 'D' and StopDate is null;";
                    if(debug) { System.out.println(query); };
                    Statement stmt = conn.createStatement();
                    stmt.executeUpdate(query);
                }

                query = "insert into IsMaintainedBy (Name, Path, Email, Kind) values ('" + dbName + 
                "', '" + dbPath + "', '" + dbaEmail + "', 'D');";
                if(debug) { System.out.println(query); };   
                Statement stmt = conn.createStatement();
                stmt.executeUpdate(query);

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to set DBA for " + dbName);
                System.err.println(se.toString());
                return "There was an error while attempting to make this assignment. The assignment was NOT made.";
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to set DBA for " + dbName);
                return "There was an error while attempting to make this assignment. The assignment was NOT made.";
            }
        } 

        return "";  //Success
    }


    /**
     * Retrieves the current DBA for the given database.
     * 
     * @param dbSource the source string of the database.
     * @return the current DBA of the database.
     */
    public String getCurrentDBA(String dbName, String dbPath)
    {
        String email = "";
        Connection conn = Connector.getConnection();
        String query = "select Email from IsMaintainedBy where Name = '" + dbName + "' and Path='" + dbPath + "' and Kind = 'D' and StopDate is null;";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {
                    email = rset.getString(1);
                    query = "Select Name from Employee where Email='" + email + "';";
                    stmt = conn.createStatement();
                    rset = stmt.executeQuery(query);

                    if(rset.next())
                    {
                        return rset.getString(1) + "(" + email + ")";
                    }
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to retrieve a DBA assignment.");
                System.err.println(se.toString());
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to retrieve a DBA assignment.");
            }
        }
        return null;
    }


    /**
     * Resets the given database's DBA so that the database does not have a DBA assigned to it.
     * @param dbName the name of the database.
     * @param dbPath the path of the database.
     */
    public void clearDBA(String dbName, String dbPath)
    {
        String query;
        String dba = getCurrentDBA(dbName, dbPath);

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                //If this is not the first DBA assigned to this database we need to update the old CSI's record with the end date
                if(dba != null)
                {
                    DateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");
                    DateFormat timeFormatter = new SimpleDateFormat("HH:mm:ss");
                    Date currentDate = new Date();

                    query = "update IsMaintainedBy set StopDate = '" + dateFormatter.format(currentDate) + " " + timeFormatter.format(currentDate) + "' where Name='" + dbName +"' and Path='" + dbPath + "' and Kind = 'D' and StopDate is null;";
                    if(debug) { System.out.println(query); };

                    Statement stmt = conn.createStatement();
                    stmt.executeUpdate(query);
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to clear DBA for " + dbName);
                System.err.println(se.toString());
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to clear DBA for " + dbName);
            }
        }
    }


    /**
     * Saves the overall configuration of the system.
     * Then it proceeds to make sure that all the databases in the system are meeting the new requirements
     * and if they aren't, it sets them to the new requirements. 
     * 
     * @param config the configuration class representing the configurations to be saved
     */
    public void setOverallSettings(DatabaseConfig config)
    {
        String query = "insert into Configuration (NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path) values (" 
            + config.getNotzIntervalDays() + ", " + config.getNotzIntervalHours() + ", " + config.getNotzIntervalMins() + ", " 
            + config.getValidationFactor() + ", '" + config.getAlgorithm() + "', 0, 'Overall', 'Overall');";
        if(debug) { System.out.println(query); };

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                stmt.executeUpdate(query);
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to save overall configuration");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to save overall configuration");
                System.err.println(e);
            }



            //****Make sure that all the databases in the system are compliant with the new settings***
            //This query gets the current configuration settings for each database in the system:
            query = "SELECT StoredDate, NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path, NotarizationScheduledDate FROM (SELECT * FROM Configuration ORDER BY StoredDate DESC) AS sorted GROUP BY Name, Path;";
            if(debug) { System.out.println(query); };

            try
            {
                boolean changed;
                int days;
                int hrs;
                int mins;
                int val;
                String alg;

                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(rset.getString(8).equals("Overall"))
                    {
                        continue;
                    }

                    changed = false;
                    days = rset.getInt(2);
                    hrs = rset.getInt(3);
                    mins = rset.getInt(4);
                    val = rset.getInt(5);
                    alg = rset.getString(6);

                    //Check days/hrs/mins
                    if(days > config.getNotzIntervalDays())   //The old day interval is too big
                    {
                        //Overwrite day, hour and minute fields with new values
                        days = config.getNotzIntervalDays();
                        hrs = config.getNotzIntervalHours();
                        mins = config.getNotzIntervalMins();
                        changed = true;
                    }
                    else if(days == config.getNotzIntervalDays()) //Days are same
                    {
                        if(hrs > config.getNotzIntervalHours())  //The old hour interval is too big
                        {
                            //Overwrite hour and minute fields with new values
                            hrs = config.getNotzIntervalHours();
                            mins = config.getNotzIntervalMins();
                            changed = true;
                        }
                        else if(hrs == config.getNotzIntervalHours())    //Days and hours are the same
                        {
                            if(mins > config.getNotzIntervalMins())  //The old minute interval is too big
                            {
                                //Overwrite minute field with new value
                                mins = config.getNotzIntervalMins();
                                changed = true;
                            }
                        }
                    }

                    //Check validation factor
                    if(val > config.getValidationFactor())
                    {
                        val = config.getValidationFactor();
                        changed = true;
                    }

                    //Check algorithm
                    if(config.getAlgorithm().equals("RGBY"))    //Alg cannot be set to monochromatic
                    {
                        if(alg.equals("Monochromatic")) //Set to RGBY
                        {
                            alg = "RGBY";
                            changed = true;
                        }
                    }
                    else if(config.getAlgorithm().equals("Tiled Bitmap"))    //Alg cannot be set to monochromatic or RGBY
                    {
                        if(alg.equals("Monochromatic") || alg.equals("RGBY")) //Set to Tiled Bitmap
                        {
                            alg = "Tiled Bitmap";
                            changed = true;
                        }
                    }
                    else if(config.getAlgorithm().equals("A3D"))    //Alg must be set to A3D
                    {
                        if(!alg.equals("A3D"))
                        {
                            alg = "A3D";
                            changed = true;
                        }
                    }

                    //If changes were made, insert a new tuple with the new settings
                    if(changed)
                    {
                        query = "insert into Configuration (NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NotarizationScheduledDate, NumLevels, Name, Path) values (" 
                            + days + ", " + hrs + ", " + mins + ", " + val + ", '" + alg + "', '" + rset.getString(10) 
                            + "', " + rset.getInt(7) + ", '" + rset.getString(8) + "', '" + rset.getString(9) + "');";
                        if(debug) { System.out.println(query); };


                        Statement stmt2 = conn.createStatement();
                        stmt2.executeUpdate(query);
                    }
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to save overall configuration");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to save overall configuration");
                System.err.println(e);
            }
        }
    }


    /**
     * Gets the current overall configuration of the system. 
     * 
     * @return the configuration of the overall system
     */
    public DatabaseConfig getCurrentOverallConfig()
    {
        Connection conn = Connector.getConnection();
        String query = "SELECT NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path FROM Configuration WHERE Name = 'Overall' ORDER BY StoredDate DESC LIMIT 1;";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {

                    return new DatabaseConfig("Overall", rset.getString(5), rset.getInt(1), rset.getInt(2), rset.getInt(3), rset.getInt(4), "0000-00-00 00:00:00");
                }

                conn.close();

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get overall configuration");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get overall configuration");
                System.err.println(e.toString());
            }
        }

        return new DatabaseConfig();
    }

    /**
     * Retrieves the list off all problems in the system. 
     * 
     * @return a vector containing the list of all problems in the system.
     */
    public Vector getAllFailures()
    {
        Vector<String> list = new Vector<String>();


        Connection conn = Connector.getConnection();
        String query = "select Name, ActualDate from Event where Kind='T';";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    list.add("Tamper detection in \"" + rset.getString(1) + "\" on " + rset.getString(2) + ".");
                }

                conn.close();

                return list;

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get all failures");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get all failures");
                System.err.println(e.toString());
            }
        }

        return list;
    }


    /**
     * Retrieves the list of all databases in the system.
     * 
     * @return a vector containing the list of databases currently in the system.
     */
    public Vector getDatabases()
    {
        Vector<String> list = new Vector<String>();
        Connection conn = Connector.getConnection();
        String query = "select distinct Name, Path from Configuration;";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(!rset.getString(1).equals("Overall"))
                    {
                        list.add(rset.getString(1) + " (" + rset.getString(2) + ")");
                    }
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get databases");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get databases");
            }
        }



        return new Vector();
    }



    /**
     * Retrieves the list of all databases managed by the DBA.
     * 
     * @return a vector containing the list of databases currently in the system.
     */
    public Vector getManagedDatabases(String dba)
    {
        StringTokenizer tokenizer = new StringTokenizer(dba, "()");
        String dbaName = tokenizer.nextToken();
        String dbaEmail = tokenizer.nextToken();

        Vector<String> list = new Vector<String>();
        Connection conn = Connector.getConnection();
        String query = "select Name from IsMaintainedBy where Email='" + dbaEmail + "' and Kind='D' and StopDate is null;";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(!rset.getString(1).equals("Overall"))
                    {
                        list.add(rset.getString(1));
                    }
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get managed databases");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get managed databases");
                System.err.println(e);
            }
        }



        return new Vector();
    }

    // add getManagedDatabasesforCSI by Michael to get the CSI managing databases
    public Vector getManagedDatabasesforCSI(String csi)
    {
        StringTokenizer tokenizer = new StringTokenizer(csi, "()");
        String csiName = tokenizer.nextToken();
        String csiEmail = tokenizer.nextToken();

        Vector<String> list = new Vector<String>();
        Connection conn = Connector.getConnection();
        String query = "select Name from IsMaintainedBy where Email='" + csiEmail + "' and Kind='C' and StopDate is null;";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(!rset.getString(1).equals("Overall"))
                    {
                        list.add(rset.getString(1));
                    }
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get managed databases");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get managed databases");
                System.err.println(e);
            }
        }



        return new Vector();
    }




    /**
     * Retrieves the list of DBAs in the system.
     * 
     * @return a vector containing all the DBAs currently in the system.
     */
    public Vector getDBAs()
    {
        Vector list = new Vector();
        Connection conn = Connector.getConnection();
        String query = "select distinct Name, Email from Employee where isDBA = 'Y';";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(!rset.getString(1).equals("Overall"))
                    {
                        list.add(rset.getString(1) + "(" + rset.getString(2) + ")");
                    }
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get DBAs");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get DBAs");
            }
        }



        return new Vector();
    }



    /**
     * Retrieves the list of CSIs in the system.
     * 
     * @return a vector containing all the CSIs currently in the system. 
     */
    public Vector getCSIs()
    {
        Vector list = new Vector();
        Connection conn = Connector.getConnection();
        String query = "select distinct Name, Email from Employee where isCSI = 'Y';";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    if(!rset.getString(1).equals("Overall"))
                    {
                        list.add(rset.getString(1) + "(" + rset.getString(2) + ")");
                    }
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get CSIs");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get CSIs");
            }
        }



        return new Vector();
    }


    /**
     * Adds a database to the system. If any errors occur while attempting to add the database
     * (either failures or duplicate names, etc) then an appropriate message is returned.
     * 
     * Creates both the initial tuple in the Configuration table for this database as well
     * as the creation event in the Event table for this database.
     * 
     * @param name the name of the database
     * @param path  the path of the database
     * @return empty string for success, otherwise a failure message
     */
    public String addDatabase(
        String name, String path) {
      String date = "";
      String time = "";
      String queryEvent;
      String line;

        //Call Huilong's program to get the start time of the database and add the event to the database
        Process p;
System.out.println("ssh " + notarize_server_ + " " + dbcreate_time_getter_ + " " + path + " " + name); 
        if(debug) { System.out.println("ssh " + notarize_server_ + " " + dbcreate_time_getter_ + " " + path + " " + name);   }
        try {
//ssh sodbvm7.cs.arizona.edu /cs/projects/tau/working/ruizhang/project/cproject/tbdb/tBDB/audit/daemons/db_create_time " + path + " " + name
            p = Runtime.getRuntime().exec("ssh " + notarize_server_ + " " + dbcreate_time_getter_ + " " + path + " " + name);
            BufferedReader in = new BufferedReader(new InputStreamReader(p.getInputStream()));

            /*
             * The output will either be:
             * Start time: 2007-03-28 11:36:54
             * or
             * Data base does not exist!
             */

            if((line = in.readLine()) != null)
            {
                if(line.equals("Data base does not exist!"))
                {
                    return "No database was found at this path with this name, please try again.";
                }

                StringTokenizer tokenizer = new StringTokenizer(line);
                tokenizer.nextToken();    //"Start"
                tokenizer.nextToken();    //"time:"
                date = tokenizer.nextToken(); //the date
                time = tokenizer.nextToken(); //the time
            }
        } catch (Exception e)
        {
            System.out.println(e);
            return "Error calculating DB start time. Database not added";
        }

        queryEvent = "INSERT INTO Event (Name, Path, Kind, ActualDate) VALUES ('" + name + "', '" + path + "', 'C', '" + date + " " + time + "');"; 


        //Get the overal settings to use as the default settings for this new db
        DatabaseConfig config = getCurrentOverallConfig();
/* Modified by Huilong */
        String query = "INSERT INTO Configuration (Name, Path, NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NotarizationScheduledDate, NumLevels) VALUES ('" + name + "', '" + path + "', '" + config.getNotzIntervalDays() + "', '" + config.getNotzIntervalHours() + "', '" + config.getNotzIntervalMins() + "', '" + config.getValidationFactor() + "', '" + config.getAlgorithm() + "', '1970-01-01 00:00:00', 0);";
/* End Huilong*/
        //String query = "INSERT INTO Configuration (Source,  NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, StartDate, StartTime, NumLevels) VALUES ('" + name + "', '" + config.getNotzIntervalDays() + "', '" + config.getNotzIntervalHours() + "', '" + config.getNotzIntervalMins() + "', '" + config.getValidationFactor() + "', '" + config.getAlgorithm() + "', null, null, null);";
        if(debug) { System.out.println(query); };

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                //Insert the Create event
                Statement stmt = conn.createStatement();
                stmt.executeUpdate(queryEvent);

                //Insert the default configuration
                stmt = conn.createStatement();
                stmt.executeUpdate(query);
                this.setChanged();
                this.notifyObservers();

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to add a database.");
                System.err.println(se.toString());
/* Added for debugging */
                System.err.println(queryEvent);
/* Qing */
                return "An unknown error has occured. Database could not be added to the system.";
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to add a database.");
                return "An unknown error has occured. Database could not be added to the system.";
            }
        }

        return "";
    }


    /**
     * Adds an employee to the system. If any errors occur while attempting to add the employee
     * (either failures or duplicate names, etc) then an appropriate message is returned.
     * 
     * Creates the tuple for the employee in the Employee table.
     * 
     * @param name the name of the employee
     * @param email the email of the employee
     * @param isCSI if this employee can be a CSI
     * @param isDBA if this employee can be a DBA
     * @param password the password for this employee
     * @param phone the phone number of this employee
     * @return blank string for success, otherwise a failure message
     */
    public String addEmployee(String name, String email, boolean isCSI, boolean isDBA, String password, String phone)
    {
        String csi, dba;

//      First check to see if this email is already in use
        Connection conn = Connector.getConnection();
        String query = "select Name from Employee where Email = '" + email + "';";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {
                    return "This email address is already in the system, please try again with a unique address.";
                }

                //Not already in database, so OK to add
                if(isCSI)
                {
                    csi = "Y";
                }
                else
                {
                    csi = "N";
                }

                if(isDBA)
                {
                    dba = "Y"; 
                }
                else
                {
                    dba = "N";
                }

                query = "INSERT INTO Employee(Name, Email, isCSI, isDBA, Password, PhoneNumber) VALUES ('" + name + "', '" + email + "', '" + csi + "', '" + dba + "', '" + password + "', '" + phone + "');";
                if(debug) { System.out.println(query); };

                stmt = conn.createStatement();
                stmt.executeUpdate(query);

                this.setChanged();
                this.notifyObservers();

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to add a database.");
                System.err.println(se.toString());
                return "An unknown error has occured. Employee could not be added to the system.";
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to add a database.");
                return "An unknown error has occured. Employee could not be added to the system.";
            }
        }

        return "";
    }
}
