package gui;
import java.sql.*;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.*;


/**
 * Maintains the state of the DBA GUI. Interacts with the database to update and
 * retrieve information about the system.
 *
 * @author Melinda Malmgren - mjmalmgr@email.arizona.edu
 *
 */
public class DBAModel extends Observable
{
    private String currentDBAName = "";
    private String currentDBAEmail = "";
    private String currentDBName = "";
    private String currentDBPath = "";

    //Set to true for debugging
    private boolean debug = false;
    
    private String notarizer_server_;
    private String binary_dir_;
    private String dns_server_;
    private String surety_password_;
    private String surety_account_name_;
    private int mode_;
    
    
    public void setDNSServer(String dns_server) {
      dns_server_ = dns_server;
    }
    
    public void setNotarizerServer(String notarizer_server) {
      notarizer_server_ = notarizer_server;
    }
    
    public void setBinaryDir(String binary_dir) {
      binary_dir_ = binary_dir;    	
    }
    
    public void setSuretyPassword(String password) {
      surety_password_ = password;
    }
      
    public void setSuretyAccountName(String account_name) {
      surety_account_name_ = account_name;
    }
      
    public void setMode(int mode) {
      mode_ = mode;
    }


    /**
     * Saves the configuration to the database.
     *
     * @param config the configuration class representing the database and configurations to be saved
     */
    public void saveDBConfig(DatabaseConfig config)
    {
        //First add a tuple to the Configuration table
        String query = "insert into Configuration (NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NotarizationScheduledDate, NumLevels, Name, Path) values (" + config.getNotzIntervalDays() + ", " + config.getNotzIntervalHours() + ", " + config.getNotzIntervalMins() + ", " + config.getValidationFactor() + ", '" + config.getAlgorithm() + "', '" + config.getStartDate() + "', 0, '" + currentDBName + "', '" + currentDBPath + "');";
        if(debug) { System.out.println(query); };

        Connection conn = Connector.getConnection();
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                stmt.executeUpdate(query);

                //Also, create the notarization event in the Event able
                query = "insert into Event (Kind, Name, Path, ScheduledDate) values ('N', '" + currentDBName + "', '" + currentDBPath + "', '" + config.getStartDate() + "');";
                if(debug) { System.out.println(query); };

                conn.createStatement();
                stmt.executeUpdate(query);

                conn.close();
            }
            catch(SQLException se)
            {
                System.err.println("SQL Exception trying to save configuration for: " + currentDBName);
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to save configuration for: " + currentDBName);
                System.err.println(e);
            }
        }

        //Call the notarizer
        Process p;
        try
        {
            if(config.getStartDate() != null)
            {
				//The at command uses a very unusual date format (YYMMDDhhmm) so we have to convert to that
				String startDate = config.getStartDate();
				startDate = startDate.substring(2,4) + startDate.substring(5,7) + startDate.substring(8,10) + startDate.substring(11,13) + startDate.substring(14,16);
				//"ssh sodbvm7.cs.arizona.edu echo \"/cs/projects/tau/working/ruizhang/project/cproject/tbdb/tBDB/audit/daemons/notarize " + currentDBPath + " " + currentDBName + " pinyon\" | at -t " + startDate
				
				String schedule_task = "ssh " + notarizer_server_ + " echo \""
                    + binary_dir_ + "/notarizer" + " "
                    + currentDBPath + " " + currentDBName + " "
                    + binary_dir_;
				if (mode_ == 0) {
			      schedule_task +=
			          " " + dns_server_ + "\" | at -t " + startDate;
				} else if (mode_ == 1) {
                  schedule_task +=
                	  " " + surety_account_name_ + " "
                	  + surety_password_ + "\" | at -t " + startDate;
				}
                if(debug) {
                  System.out.println(schedule_task);
                }
                p = Runtime.getRuntime().exec(schedule_task);
                p.waitFor();
                p.destroy();
            }
        }
        catch (Exception e)
        {
			System.out.println(e);
        }
    }



    /**
     * Retrieves the current configuration settings from the SMDB for
     * the currently selected database.
     *
     * @return the configuration settings of the currently selected database
     */
    public DatabaseConfig getCurrentConfig()
    {
        if(currentDBPath == null || currentDBName == null || currentDBPath.equals("") || currentDBName.equals(""))
        {
            return new DatabaseConfig();
        }

        Connection conn = Connector.getConnection();
        String query = "select NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, NotarizationScheduledDate from Configuration where Name = '" + currentDBName + "' and Path = '" + currentDBPath + "' order by StoredDate desc;";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {
                    //String source, String algorithm, int nIntD, int nIntH, int nIntM, int validationFactor, String date
                    return new DatabaseConfig(currentDBName, rset.getString(5), rset.getInt(1), rset.getInt(2), rset.getInt(3), rset.getInt(4), rset.getString(7));
                }

                conn.close();

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get db configuration : ");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get db configuration");
                System.err.println(e.toString());
            }
        }

        return new DatabaseConfig();
    }


    /**
     * Retrieves a list of all the failures associated with the current database.
     *
     * @return the list of problems associated with this database
     */
    public ArrayList getFailures()
    {
        ArrayList<String> list = new ArrayList<String>();

        if(currentDBPath == null || currentDBName == null || currentDBPath.equals("") || currentDBName.equals(""))
        {
            return list;
        }

        Connection conn = Connector.getConnection();
        String query = "select Name, ActualDate from Event where Name = '" + currentDBName + "' and Kind='T';";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                while(rset.next())
                {
                    list.add("Tamper detection in \"" + rset.getString(1) + "\" on " + rset.getString(2) + ".");
                }

                conn.close();

                return list;

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get all failures");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get all failures");
                System.err.println(e.toString());
            }
        }

        return list;
    }



    /**
     * Sets the currently selected database.
     *
     * @param dbSource the source string of the database to search for.
     */
    public void setSelectedDB(String dbName, String dbPath)
    {
        currentDBName = dbName;
        currentDBPath = dbPath;
        this.setChanged();
        this.notifyObservers();
    }

    public String getSelectedDB()
    {
        return currentDBName;
    }

    /**
     * Sets the currently logged in DBA
     *
     * @param dbSource the source string of the database to search for.
     */
    public void setCurrentDBA(String email)
    {
        currentDBAEmail = email;
    }



    /**
     * Retrieves a list of all the databases that the given DBA is assigned to.
     * Changes the currently selected database to be the first database in this list.
     *
     * @return a vector containing all the source strings of the databases.
     */
    public Vector getManagedDatabases()
    {
        if(currentDBAName == null || currentDBAEmail == null)
        {
            return new Vector();
        }

        Vector list = new Vector();
        Connection conn = Connector.getConnection();
        String query = "select Name, Path from IsMaintainedBy where Email='" + currentDBAEmail + "' and Kind='D' and StopDate is null;";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next()) //First database will be selected
                {
                    currentDBName = rset.getString(1);
                    currentDBPath = rset.getString(2);
                    list.add(currentDBName + "(" + currentDBPath + ")");
                    //this.setChanged();
                    //this.notifyObservers();
                }

                while(rset.next())
                {
                    list.add(rset.getString(1) + "(" + rset.getString(2) + ")");
                }

                conn.close();
                return list;
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get managed databases");
                System.err.println(se);
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get managed databases");
            }
        }
        return new Vector();
    }

    /**
     * Gets the current overall configuration of the system.
     *
     * @return the configuration of the overall system
     */
    public DatabaseConfig getCurrentOverallConfig()
    {
        Connection conn = Connector.getConnection();
        String query = "SELECT NotIntDays, NotIntHrs, NotIntMins, ValidationFactor, Algorithm, NumLevels, Name, Path FROM Configuration WHERE Name = 'Overall' ORDER BY StoredDate DESC LIMIT 1;";
        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {

                    return new DatabaseConfig("Overall", rset.getString(5), rset.getInt(1), rset.getInt(2), rset.getInt(3), rset.getInt(4), "0000-00-00 00:00:00");
                }

                conn.close();

            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to get overall configuration");
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to get overall configuration");
                System.err.println(e.toString());
            }
        }

        return new DatabaseConfig();
    }


    /**
     * Gets the current CSI for the currently selected database. Returns a blank string if there is
     * no CSI or if a database is not selected.
     *
     * @return the CSI of the given database in the form: Name(email)
     */
    public String getCurrentCSI()
    {
        String email = "";
        Connection conn = Connector.getConnection();
        String query = "select Email from IsMaintainedBy where Name = '" + currentDBName + "' and Path='" + currentDBPath + "' and Kind='C' and StopDate is null;";
        if(debug) { System.out.println(query); };

        if(conn != null)
        {
            try
            {
                Statement stmt = conn.createStatement();
                ResultSet rset = stmt.executeQuery(query);

                if(rset.next())
                {
                    email = rset.getString(1);
                    query = "Select Name from Employee where Email='" + email + "';";
                    stmt = conn.createStatement();
                    rset = stmt.executeQuery(query);

                    if(rset.next())
                    {
                        return rset.getString(1) + "(" + email + ")";
                    }
                }

                conn.close();
            }
            catch (SQLException se)
            {
                System.err.println("SQL Exception trying to retrieve a CSI assignment.");
                System.err.println(se.toString());
            }
            catch(Exception e)
            {
                System.err.println("Exception trying to retrieve a CSI assignment.");
            }
        }
        return "";
    }

}
