package gui;
import java.sql.Timestamp;

/**
 * Represents a database configuration. 
 * 
 * @author Natasha Gaitonde - gaitonde@email.arizona.edu
 *
 */
public class DatabaseConfig
{
	private String dbName = "";
    private String algorithm = "";
    private int notzIntervalHours = 0;
    private int notzIntervalDays = 0;
    private int notzIntervalMins = 0;
    private int validationFactor = 0;
    private int level = 0;
    private Timestamp storedDate = null;
    private String date = "";

private String dbaName = "";
    private String csiName = "";
    private String source = "";
    private String startDate = "";


    /**
     * Blank constructor for creating blank configurations.
     */
    public DatabaseConfig()
    {}

    /**
     * @param dbName the name of the database whose configuration is in this object
     * @param algorithm the forensic algorithm to be used
     * @param nIntD the notarization interval in days to be used
     * @param nIntH the notarization interval in hours to be used
     * @param nIntM the notarization interval in minutes to be used
     * @param validationFactor the validation factor to be used
     * @param level the level to be used
     * @param storedDate the time at which the configuration was/is created
     * @param date the NotarizationScheduledDate to be used
     */
    public DatabaseConfig(String dbName, String algorithm, int nIntD, int nIntH, int nIntM, int validationFactor, int level, Timestamp storedDate, String date)
    {
    	this.dbName=dbName;
        this.algorithm = algorithm;
        this.notzIntervalDays = nIntD;
        this.notzIntervalHours = nIntH;
        this.notzIntervalMins = nIntM;
        this.validationFactor = validationFactor;
        this.level=level;
        this.storedDate=new Timestamp(storedDate.getTime());
        this.date = date;
    }

    //Determine if this object is the same as another DatabaseConfig object
    boolean equalsObj(DatabaseConfig confignew)
    {
    	if(this.algorithm.trim().equalsIgnoreCase(confignew.algorithm.trim()))
    	{
    		if(this.level==confignew.level)
    		{
    			if(this.notzIntervalDays==confignew.notzIntervalDays)
    			{
    				if(this.notzIntervalHours==confignew.notzIntervalHours)
    				{
    					if(this.notzIntervalMins==confignew.notzIntervalMins)
    					{
    						if(this.validationFactor==confignew.validationFactor)
    						{
    							return false;
    						}
    					}
    				}
    			}    			
    		}
    	}
    	return true;
    }
    
    /**
     * @return the current algorithm
     */
    public String getAlgorithm()
    {
        return algorithm;
    }

    /**
     * @param algorithm the new algorithm
     */
    public void setAlgorithm(String algorithm)
    {
        this.algorithm = algorithm;
    }

    /**
     * @return the current notarization interval in days
     */
    public int getNotzIntervalDays()
    {
        return notzIntervalDays;
    }

    /**
     * @param notzIntervalDays the new notarization interval in days
     */
    public void setNotzIntervalDays(int notzIntervalDays)
    {
        this.notzIntervalDays = notzIntervalDays;
    }

    /**
     * @return the current notarization interval in hours
     */
    public int getNotzIntervalHours()
    {
        return notzIntervalHours;
    }

    /**
     * @param notzIntervalHours the new notarization interval in hours
     */
    public void setNotzIntervalHours(int notzIntervalHours)
    {
        this.notzIntervalHours = notzIntervalHours;
    }

    /**
     * @return the current notarization interval in minutes
     */
    public int getNotzIntervalMins()
    {
        return notzIntervalMins;
    }

    /**
     * @param notzIntervalMins the new notarization interval in minutes
     */
    public void setNotzIntervalMins(int notzIntervalMins)
    {
        this.notzIntervalMins = notzIntervalMins;
    }

    /**
     * @return the current start date
     */
    public String getDate()
    {
        return date;
    }

    /**
     * @param startDate the new start date
     */
    public void setDate(String date)
    {
        this.date = date;
    }

    /**
     * @return the current validation factor
     */
    public int getValidationFactor()
    {
        return validationFactor;
    }

    /**
     * @param validationFactor the new validation factor
     */
    public void setValidationFactor(int validationFactor)
    {
        this.validationFactor = validationFactor;
    }

    public int getLevel()
    {
        return level;
    }

    /**
     * @param level the new level
     */
    public void setLevel(int level)
    {
        this.level = level;
    }
    
    /**
     * Prints the current configuration to standard out
     */
    public void print()
    {
        System.out.println("Algorithm = " 
                + algorithm + "\nStoredDate = " + storedDate.toString() + " Notarization Interval = " + notzIntervalDays + " days " + notzIntervalHours
                + " hrs " + notzIntervalMins + " mins\nValidation Factor = " + validationFactor 
                + "\nStart Date = " + date + "\nLevel = " + level);

        System.out.println("Source = " + source + "\nDBA = " + dbaName + "\nCSI = " + csiName + "\nAlgorithm = " 
                + algorithm + "\nNotarization Interval = " + notzIntervalDays + " days " + notzIntervalHours
                + " hrs " + notzIntervalMins + " mins\nValidation Factor = " + validationFactor 
                + "\nStart Date = " + startDate);
    }

	public Timestamp getStoredDate() 
	{
		return storedDate;
	}

	public void setStoredDate(Timestamp storedDate) 
	{
		this.storedDate=null;
		this.storedDate = new Timestamp(storedDate.getTime());
	}



    /**
     * @param algorithm the forensic algorithm to be used
     * @param nIntD the notarization interval in days to be used
     * @param nIntH the notarization interval in hours to be used
     * @param nIntM the notarization interval in minutes to be used
     * @param validationFactor the validation factor to be used
     */
    public DatabaseConfig(String algorithm, int nIntD, int nIntH, int nIntM, int validationFactor)
    {
        this.algorithm = algorithm;
        this.notzIntervalDays = nIntD;
        this.notzIntervalHours = nIntH;
        this.notzIntervalMins = nIntM;
        this.validationFactor = validationFactor;
    }

    /**
     * Intializes with everything except the CSI and DBA.
     * 
     * @param source the source of the database
     * @param algorithm the forensic algorithm to be used
     * @param nIntD the notarization interval in days to be used
     * @param nIntH the notarization interval in hours to be used
     * @param nIntM the notarization interval in minutes to be used
     * @param validationFactor the validation factor to be used
     * @param date the date on which to start using these new configurations
     */
    public DatabaseConfig(String source, String algorithm, int nIntD, int nIntH, int nIntM, int validationFactor, String date)
    {
        this(algorithm, nIntD, nIntH, nIntM, validationFactor);
        this.source = source;
        this.startDate = date;
    }

    /**
     * Initializes everything.
     * 
     * @param source the source of the database
     * @param dba the name of the DBA for this database
     * @param csi the name of the CSI for this database
     * @param algorithm the forensic algorithm to be used
     * @param nIntD the notarization interval in days to be used
     * @param nIntH the notarization interval in hours to be used
     * @param nIntM the notarization interval in minutes to be used
     * @param validationFactor the validation factor to be used
     * @param date the date on which to start using these new configurations
     */
    public DatabaseConfig(String source, String dba, String csi, String algorithm, int nIntD, int nIntH, int nIntM, int validationFactor, String date)
    {
        this(source, algorithm, nIntD, nIntH, nIntM, validationFactor, date);
        this.dbaName = dba;
        this.csiName = csi;
    }




    /**
     * @return the name of the CSI
     */
    public String getCSIName()
    {
        return csiName;
    }

    /**
     * @param csiName the name of the CSI
     */
    public void setCSIName(String csiName)
    {
        this.csiName = csiName;
    }

    /**
     * @return the name of the DBA
     */
    public String getDBAName()
    {
        return dbaName;
    }

    /**
     * @param dbaName the name of the DBA
     */
    public void setDBAName(String dbaName)
    {
        this.dbaName = dbaName;
    }

    /**
     * @return the current source
     */
    public String getSource()
    {
        return source;
    }

    /**
     * @param source the source string of the database
     */
    public void setSource(String source)
    {
        this.source = source;
    }

    /**
     * @return the current start date
     */
    public String getStartDate()
    {
        return startDate;
    }

    /**
     * @param startDate the new start date
     */
    public void setStartDate(String startDate)
    {
        this.startDate = startDate;
    }

    /**
     * @return the current validation factor
     */


}
