package gui;
/******************************************************/
/* Original Author: Kyri Pavlou         1 August 2008 */
/******************************************************/

public class ForensicCost {
	/*This function evaluates the logarithm of 'num' of base 'base' */
	double log_base_n(double num, double base )
	{
	    double  res;
	    if(num < 1 )
	    {
	        res = 0;
	    }
	    else 
	    {
	        res = Math.log(num)/Math.log(base);
	    }
	    return res;
	}

	/*****************************************************************************/
	/*Recursive function to generate the value of $\mathcal{N}(D)$ the first     */
	/*component of the Total number of notarizations                             */
	/*-- see section 8.4: The a3D Algorithm                                      */
	/*in "Forensic Analysis of Database Tampering"                               */
	/*D is the number of days/granules before the first validation failure       */
	/*****************************************************************************/
	double N_rec(int D)
	{
	    if(D % 2 == 1)
	        return N_rec(D-1);
	    else
	    {
	        double Nres; 
	        int i;
	        if(D == 0)
	           return 0;
	        Nres = 0;
	        for(i = 0 ; i <= (int)Math.floor(log_base_n(D,2)) ; i ++)
	        {
	            if( (D & (1 << (i))) != 0)
	                Nres += ((1 << (i+1)) - 1);
	        }
	        return Nres;
	    }
	}

	/*****************************************************************************/
	/*Function to evaluate the Normal_Processing component of the                */
	/*forensic cost for the monochromatic algorithm                              */
	/* D is the number of days/granules before the first validation failure      */
	/* V is the validation factor (I_V= V * I_N)                                 */
	/*****************************************************************************/
	double NP_mono(double D, double V)
	{
	    return  D + D/V;
	}

	/*****************************************************************************/
	/*Function to evaluate the Forensic_Analysis component                       */
	/*of the forensic cost for the monochromatic algorithm                       */
	/* D is the number of days/granules before the first validation failure      */
	/*****************************************************************************/
	double FA_mono(double D)
	{
	    return 2*Math.ceil(log_base_n(D, 2));
	}

	/*****************************************************************************/
	/*Function to evaluate the Normal_Processing component of the                */
	/*forensic cost for the RGBY algorithm                                       */
	/* D is the number of days/granules before the first validation failure      */
	/*****************************************************************************/
	double NP_RGBY(double D)
	{
	    return D + 3*(D/2);
	}

	/*****************************************************************************/
	/*Function to evaluate the Forensic_Analysis component                       */
	/*of the forensic cost for the RGBY algorithm                                */
	/* D is the number of days/granules before the first validation failure      */
	/*****************************************************************************/
	double FA_RGBY(double D)
	{
	    return 2*log_base_n(D, 2) +  2*(D/2);
	}

	/*****************************************************************************/
	/*Function to evaluate the Normal_Processing component of the                */
	/*forensic cost for the tiled bitmap algorithm                               */
	/* D is the number of days/granules before the first validation failure      */
	/* N is the notarization factor (I_N = N * R_s)                              */
	/* V is the validation factor (I_V = V * I_N)                                */
	/*****************************************************************************/
	double NP_tiled(double D, double N, double V)
	{
	    return (D / N) + (1 + log_base_n(N, 2))*(D/N) +  (D/(V*N));
	}

	/*****************************************************************************/
	/*Function to evaluate the Forensic_Analysis component                       */
	/*of the forensic cost for the tiled bitmap algorithm                        */
	/* D is the number of days/granules before the first validation failure      */
	/* N is the notarization factor (I_N = N * R_s)                              */
	/* k is the number of corruption sites                                       */
	/*****************************************************************************/
	double FA_tiled(double D, double N, int k)
	{
	    return 2*log_base_n(D/N, 2) + (1 + log_base_n(N, 2))*k;
	}

	/*****************************************************************************/
	/*Function to evaluate the Normal_Processing component of the                */
	/*forensic cost for the a3D algorithm                                        */
	/* D is the number of days/granules before the first validation failure      */
	/* N is the notarization factor                                              */
	/*****************************************************************************/
	double NP_a3D(double D, double N)
	{
	     return   (D/N) + N_rec((int)D) + (D/N) - (1 + Math.floor(log_base_n((D/N),2)));
	}

	/*****************************************************************************/
	/*Function to evaluate the Forensic_Analysis component                       */
	/*of the forensic cost for the a3D algorithm                                 */
	/* D is the number of days/granules before the first validation failure      */
	/* k is the number of corruption sites                                       */
	/*****************************************************************************/
	double FA_a3D(double D, int k)
	{
	    //this overestimates the number of validations because it assumes a complete tree
	    if( 1 <=k || k <= D/2 )
	        return 2*k*(Math.ceil(log_base_n(D,2)) - Math.ceil(log_base_n(k,2)))
	                + (1 + Math.ceil(log_base_n(k,2)) - Math.floor(log_base_n(k,2)))
	                *(1<< ((int)Math.floor(log_base_n(k,2)) + 1)) - 1;
	    else // k > D/2
	        return 2*D-1;
	}
}
